@@ -1,15 +1,14 @@
-package # hide from PAUSE
-Sys::CpuAffinity;
+package Sys::CpuAffinity;
 use Carp;
 use warnings;
 use strict;
 use base qw(DynaLoader);
 
-our $VERSION = '0.96';
+our $VERSION = '0.98';
+our $DEBUG = $ENV{DEBUG} || 0;
 eval { bootstrap Sys::CpuAffinity $VERSION };
 
 sub import {
-  #reserved for future use
 }
 
 sub _maskToArray {
@@ -119,17 +118,17 @@ sub setAffinity {
   }
 
   return _setAffinity_with_Win32API($pid,$mask)
+    || _setAffinity_with_xs_win32($pid,$mask)
     || _setAffinity_with_Win32Process($pid,$mask)
     || _setAffinity_with_taskset($pid,$mask)
     || _setAffinity_with_xs_sched_setaffinity($pid,$mask)
     || _setAffinity_with_BSD_Process_Affinity($pid,$mask)
-    || _setAffinity_with_xs_cpuset_setaffinity($pid,$mask) # XXX needs work
+    || _setAffinity_with_xs_cpuset_setaffinity($pid,$mask)  # XXX needs wor
     || _setAffinity_with_xs_processor_bind($pid,$mask)
     || _setAffinity_with_bindprocessor($pid,$mask)
     || _setAffinity_with_cpuset($pid,$mask)
     || _setAffinity_with_pbind($pid,$mask)
-#   || _setAffinity_with_psrset($pid,$mask)       # XXX needs proper test env
-    || _setAffinity_with_xs_win32($pid,$mask)
+#   || _setAffinity_with_psrset($pid,$mask) # XXX needs proper test e
     || 0;
 }
 
@@ -176,12 +175,14 @@ sub _getNumCpus_from_ENV {
 }
 
 sub _getNumCpus_from_Win32API {
-  # GetActiveProcessorCount api function is only supported since Windows 7
+  # GetActiveProcessorCount api function is only supported since Windows 7?
+  # !!! Unfortunately, it also seems to make Windows 7 crash !!!
   return 0 if $^O ne "MSWin32" && $^O ne "cygwin";
   return 0 if !_configModule("Win32::API");
 
   # ALL_PROCESSOR_GROUPS: 0xffff
-  return _win32api("GetActiveProcessorCount", 0xffff) || 0;
+  ### return _win32api("GetActiveProcessorCount", 0xffff) ||
+    0;
 }
 
 our %WIN32_SYSTEM_INFO = ();
@@ -199,6 +200,8 @@ sub _getNumCpus_from_Win32API_System_Info {
         $lpsysinfo_type_avail ? 'LPSYSTEM_INFO' : 'PCHAR';
       $WIN32API{"GetSystemInfo"} = Win32::API->new('kernel32', $proto);
     }
+
+    # does this part break on 64-bit machines?
     my $buffer = chr(0) x 36;
     $WIN32API{"GetSystemInfo"}->Call($buffer);
     ($WIN32_SYSTEM_INFO{"PageSize"},
@@ -437,8 +440,8 @@ sub _getAffinity_with_Win32API {
   return 0 if !_configModule("Win32::API");
 
   my $pid = $opid;
-  if ($^O eq "cygwin" && defined &Cygwin::pid_to_winpid) {
-    $pid = Cygwin::pid_to_winpid($opid);
+  if ($^O eq "cygwin") {
+    $pid = __pid_to_winpid($opid);
   }
 
   if ($pid > 0) {
@@ -508,8 +511,8 @@ sub _getAffinity_with_Win32Process {
   return 0 if !_configModule("Win32::Process");
   return 0 if $pid < 0;  # pseudo-process / thread id
 
-  if ($^O eq "cygwin" && defined &Cygwin::pid_to_winpid) {
-    $pid = Cygwin::pid_to_winpid($pid);
+  if ($^O eq "cygwin") {
+    $pid = __pid_to_winpid($pid);
   }
 
   my ($processHandle, $processMask, $systemMask, $result);
@@ -572,8 +575,8 @@ sub _getAffinity_with_pbind {
 
 sub _getAffinity_with_xs_processor_bind {
   my ($pid) = @_;
-  return 0 if !defined &xs_getaffinity_process_bind;
-  my $mask = xs_getaffinity_process_bind($pid);
+  return 0 if !defined &xs_getaffinity_processor_bind;
+  my $mask = xs_getaffinity_processor_bind($pid);
   if ($mask == -10) {
     my $np = getNumCpus();
     if ($np > 0) {
@@ -635,8 +638,8 @@ sub _getAffinity_with_xs_cpuset_getaffinity {
 sub _getAffinity_with_xs_win32 {
   my ($opid) = @_;
   my $pid = $opid;
-  if ($^O =~ /cygwin/ && defined &Cygwin::pid_to_winpid) {
-    $pid = Cygwin::pid_to_winpid($opid);
+  if ($^O =~ /cygwin/) {
+    $pid = __pid_to_winpid($opid);
   }
   if ($pid < 0) {
     return 0 if !defined &xs_win32_getAffinity_thread;
@@ -665,14 +668,22 @@ sub _setAffinity_with_Win32API {
 
   # if $^O is "cygwin", make sure you are passing the Windows pid,
   # using Cygwin::pid_to_winpid if necessary!
-  if ($^O eq "cygwin" && defined &Cygwin::pid_to_winpid) {
-    $pid = Cygwin::pid_to_winpid($pid);
+
+  if ($^O eq "cygwin") {
+    $pid = __pid_to_winpid($pid);
+    if ($DEBUG) {
+      print STDERR "winpid is $pid ($_[0])\n";
+    }
   }
 
   if ($pid > 0) {
     my $processHandle;
     # 0x0200 - PROCESS_SET_INFORMATION
-    return 0 unless $processHandle = _win32api("OpenProcess", 0x0200,0,$pid);
+    $processHandle = _win32api("OpenProcess", 0x0200,0,$pid);
+    if ($DEBUG) {
+      print STDERR "process handle: $processHandle\n";
+    }
+    return 0 unless $processHandle;
     my $result = _win32api("SetProcessAffinityMask", $processHandle, $mask);
     _debug("set affinity with Win32::API: $result");
     return $result;
@@ -706,13 +717,46 @@ sub _setAffinity_with_Win32Process {
   my ($pid, $mask) = @_;
   return 0 if $^O ne "MSWin32" && $^O ne "cygwin";
   return 0 if !_configModule("Win32::Process");
-  if ($^O eq "cygwin" && defined &Cygwin::pid_to_winpid) {
-    $pid = Cygwin::pid_to_winpid($pid);
+
+  $DB::single = 1;
+
+  if ($^O eq "cygwin") {
+    $pid = __pid_to_winpid($pid);
+
+    if ($DEBUG) {
+      print STDERR "cygwin pid $_[0] => winpid $pid\n";
+    }
   }
 
   my $processHandle;
   return 0 unless Win32::Process::Open($processHandle, $pid, 0)
     && ref $processHandle eq 'Win32::Process';
+
+  # Seg fault on Cygwin? We really prefer not to use it on Cygwin.
+  local $SIG{SEGV} = 'IGNORE';
+
+  # SetProcessAffinityMask: "only available on Windows NT"
+  use Config;
+  my $v = $Config{osvers};
+  if ($^O eq 'MSWin32' && ($v < 3.51 || $v >= 6.0)) {
+    if ($DEBUG) {
+      print STDERR "SetProcessAffinityMask ",
+	"not available on MSWin32 osvers $v?\n";
+    }
+    return 0;
+  }
+  # Don't trust Strawberry Perl $Config{osvers}. Win32::GetOSVersion
+  # is more reliable if it is available.
+  if (_configModule('Win32')) {
+    if (!Win32::IsWinNT()) {
+      if ($DEBUG) {
+	print STDERR "SetProcessorAffinityMask ",
+	  "not available on MSWin32 OS Version $v\n";
+      }
+      return 0;
+    }
+  }
+
   my $result = $processHandle->SetProcessAffinityMask($mask);
   _debug("set affinity with Win32::Process: $result");
   return $result;
@@ -777,7 +821,7 @@ sub _setAffinity_with_xs_processor_bind {
     return $result;
   } else {
     my @amask = _maskToArray($mask);
-    return 0 if !defined &xs_setaffinity_process_bind;
+    return 0 if !defined &xs_setaffinity_processor_bind;
 
     # solaris processor_bind() is for binding to a single processor.
     # see comment under _setAffinity_with_pbind().
@@ -847,9 +891,12 @@ sub _setAffinity_with_xs_cpuset_setaffinity {
 
 sub _setAffinity_with_xs_win32 {
   my ($opid, $mask) = @_;
+
+$DB::single = 1;
+
   my $pid = $opid;
   if ($^O =~ /cygwin/) {
-    $pid = Cygwin::pid_to_winpid($opid);
+    $pid = __pid_to_winpid($opid);
   }
 
   if ($pid < 0) {
@@ -859,7 +906,8 @@ sub _setAffinity_with_xs_win32 {
     }
     return 0;
   } elsif ($opid == $$) {
-    if (defined &xs_win32_setAffinity_thread) {
+
+    if ($^O ne 'cygwin' && defined &xs_win32_setAffinity_thread) {
       my $r = xs_win32_setAffinity_thread(0, $mask);
       return $r if $r;
     }
@@ -867,6 +915,10 @@ sub _setAffinity_with_xs_win32 {
       _debug("xs_win32_setAffinity_proc \$\$");
       return xs_win32_setAffinity_proc($pid,$mask);
     }
+    if ($^O eq 'cygwin' && defined &xs_win32_setAffinity_thread) {
+      my $r = xs_win32_setAffinity_thread(0, $mask);
+      return $r if $r;
+    }
     return 0;
   } elsif (defined &xs_win32_setAffinity_proc) {
     _debug("xs_win32_setAffinity_proc +\$pid");
@@ -875,11 +927,34 @@ sub _setAffinity_with_xs_win32 {
   return 0;
 }
 
+sub __pid_to_winpid {
+  my ($cygwinpid) = @_;
+  if ($] >= 5.008 && defined(&Cygwin::pid_to_winpid)) {
+    return Cygwin::pid_to_winpid($cygwinpid);
+  } else {
+    return __poor_mans_pid_to_winpid($cygwinpid);
+  }
+}
+
+sub __poor_mans_pid_to_winpid {
+  my ($cygwinpid) = @_;
+  my @psw = `/usr/bin/ps -W`;
+  foreach my $psw (@psw) {
+    $psw =~ s/^[A-Z\s]+//;
+    my ($pid,$ppid,$pgid,$winpid) = split /\s+/, $psw;
+    next unless $pid;
+    if ($pid == $cygwinpid) {
+      return $winpid;
+    }
+  }
+  warn "Could not resolve cygwin pid $cygwinpid into winpid.\n";
+  return $cygwinpid;
+}
+
 ######################################################################
 
 # configuration code
 
-our $DEBUG = $ENV{DEBUG} || 0;
 sub _debug {
   return if !$DEBUG;
   print STDERR "Sys::CpuAffinity: ",@_,"\n";
@@ -917,6 +992,7 @@ sub _configExternalProgram {
 
     if ($which =~ / not in / 			# negative output on irix
 	|| $which =~ /no \Q$program\E in /	# negative output on solaris
+	|| $which =~ /Command not found/        # negative output on openbsd
        ) {
 
       $which = '';
@@ -998,7 +1074,7 @@ sub _win32api {
       return;
     }
   }
-  return if !defined $WIN32API{$function} || $WIN32API{$function} == 0;
+  return if !defined($WIN32API{$function}) || $WIN32API{$function} == 0;
   return $WIN32API{$function}->Call(@_);
 }
 
@@ -1016,7 +1092,7 @@ Sys::CpuAffinity - Set CPU affinity for processes
 
 =head1 VERSION
 
-Version 0.96
+Version 0.98
 
 =head1 SYNOPSIS
 
@@ -1103,7 +1179,7 @@ So for example, if a process in an 8-CPU machine
 had affinity for CPU's # 2, 6, and 7, then
 in scalar context, C<getAffinity()> would return
 
-    (1 << 2) || (1 << 6) | (1 << 7) ==> 196
+    (1 << 2) | (1 << 6) | (1 << 7) ==> 196
 
 and in array context, it would return
 
@@ -1213,11 +1289,11 @@ the number of processors.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2010, Marty O'Brien.
+Copyright 2010 Marty O'Brien.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.8 or,
-at your option, any later version of Perl 5 you may have available.
+This program is free software; you can redistribute it and/or modify it
+under the terms of either: the GNU General Public License as published
+by the Free Software Foundation; or the Artistic License.
 
 See http://dev.perl.org/licenses/ for more information.
 
@@ -1307,3 +1383,13 @@ by default.
 Some systems have a concept of the maximum number of processors that
 they can suppport.
 
+Currently (0.91-0.98), constant parameters to Win32 API functions are 
+hard coded, not extracted from the local header files. Microsoft is
+probably loathe to change these constants between different versions,
+but this still seems dodgy.
+
+
+##########################################
+
+Cygwin: if Win32::API is not installed and setCpuAffinity doesn't work,
+recommend Win32::API
